* COV_ENG1.PRG
* >L<

* Creates a subclass of cov_engine rather
* than the cov_standard, as ones in 
* COV_SUBS.VCX, all do.

* This subclass gets its interface from a form class very similar
* to the one used as an AddIn elsewhere. (Add_Log.PRG, in which
* it shows that you can have more than one log being
* analysed at the same time.) The point is that
* the engine does all the work and the interface can
* be whatever you want.  COV_ENG2.PRG is a second
* engine subclass, which gets its interface from GR_STATS.SCX.

* Read ADD_LOG.TXT in the AddIns sample folder for more information
* about this dialog interface in this silly little class.

* There is no significance to the fact that this
* and COV_ENG2.PRG are PRGs rather than in a VCX, except 
* they were both written as quickly as possible <g>.

LPARAMETERS tcFile, tlUnattended, tcAddIn
IF NOT FILE("COVERAGE.VCX")
   MESSAGEBOX("You need to either:"+CHR(13)+CHR(13)+ ;
              "*-- build this PRG into an APP or "+;
              CHR(13)+CHR(13)+ ;
              "*-- SET PATH TO <folder containing COVERAGE.VCX>"+;
              CHR(13)+CHR(13)+ ;
              "before running this demo program!",16,;
              "This Coverage Engine Subclass demo can't run")
   RETURN .F.
ENDIF              

LOCAL lcLibs
lcLibs = SET("CLASSLIB")
SET CLASSLIB TO COVERAGE ADDITIVE
CREATEOBJECT("cov_EngineSubClass", tcFile, tlUnattended, tcAddIn)
SET CLASSLIB TO &lcLibs
RETURN

DEFINE CLASS cov_EngineSubClass AS cov_Engine

   PROC Init(tcFile,tlUnattended, tcAddIn)
      LOCAL llReturn
      llReturn = DODEFAULT(tcFile,tlUnattended, tcAddIn)
      IF (NOT llReturn) OR THIS.lUnattended
         THIS.Release()
         * to get rid of public reference etc;
         * in this limited class,
         * we don't want it lying around
         * whereas in cov_standard and
         * some other times we may 
         * interrogate the object before
         * getting rid of it, in a wrapper program
      ENDIF
   RETURN llReturn
   
   PROC CreateForms
     LOCAL llReturn
     llReturn = DODEFAULT()
     IF llReturn
        * the formset exists in the 
        * engine only invisibly.
        * you decide when, and if,
        * you make any components of a UI
        * visible.
        * If you're running unattended, you never
        * bother with this.
        IF THIS.lInCoverageFrame
           THIS.oFrame.Show()
        ENDIF
        THIS.AddObject("MyForm","MyFormClass")
        IF THIS.lError
           llReturn = .F.
        ENDIF
     ENDIF
   RETURN llReturn
   
   PROC SetUIToShowCode(tcSource, tcTarget)
      IF DODEFAULT(tcSource, tcTarget)
         LOCAL lcTarget
         lcTarget = THIS.GetAlias(tcTarget,"Target")
         THIS.MyForm.SetSources(lcTarget)
         THIS.MyForm.Show()
      ELSE
         RETURN .F.
      ENDIF
   ENDPROC

ENDDEFINE

DEFINE CLASS MyFormClass AS Form
   Left = 0
   Top  = 0
   ShowWindow = 1
   Name = "frmCoverage"
   cMarkedCodeField = ""
   nMargin = SYSMETRIC(3)
   MinHeight = THIS.Height
   MinWidth = THIS.Width
  
   ADD OBJECT lstAvailableObjectsAndProcs AS listbox WITH ;
      Name = "lstAvailableObjectsAndProcs", ;
      DragMode = 1, ;
      RowSourceType = 2, ;
      ColumnCount = 2, ;
      Left = THIS.nMargin, ;         
      Top = THIS.nMargin, ;
      Width = THIS.Width - (THIS.nMargin*2)


   ADD OBJECT edtMarkedCode AS editbox WITH ;
      Name = "edtMarkedCode", ;
      DragMode = 1, ;
      Left = THIS.nMargin, ;         
      Top = THIS.nMargin * 2 + THIS.lstAvailableObjectsAndProcs.Height, ;
      Width = THIS.Width - (THIS.nMargin*2)

   PROC Init
      THIS.SetInitialFormProperties()
      THIS.Resize()
   ENDPROC
      
   PROC lstAvailableObjectsAndProcs.DragDrop(oSource,nXCoord,nYCoord)
      THISFORM.DragDrop(oSource, nXCoord, nYCoord)
   ENDPROC
   
   PROC edtMarkedCode.DragDrop(oSource, nXCoord, nYCoord)
      THISFORM.DragDrop(oSource, nXCoord, nYCoord)
   ENDPROC
 
   PROC DragDrop(oSource, nXCoord, nYCoord)
   
      DO CASE
      CASE UPPER(oSource.Name) == ;
           UPPER(THIS.Controls(1).Name) ;
           AND ;
           BETWEEN(nYCoord, ;
                   THIS.nMargin, ;
                   THIS.Height - (THIS.nMargin * 7))

         THIS.lstAvailableObjectsAndProcs.Height = nYCoord

         THIS.edtMarkedCode.Top = ;            
              (THIS.lstAvailableObjectsAndProcs.Height + ;
               (THIS.nMargin * 2))
              
         THIS.edtMarkedCode.Height = ;
              THIS.Height - ;
              (THIS.edtMarkedCode.Top + ;
               THIS.nMargin )

      CASE UPPER(oSource.Name) == ;
           UPPER(THIS.Controls(2).Name) ;
           AND ;
           BETWEEN(nYCoord, ;
                 THIS.nMargin * 4, ;
                 THIS.Height - (THIS.nMargin * 3))

         THIS.edtMarkedCode.Top = nYCoord
         THIS.edtMarkedCode.Height = ;
              THIS.Height - (nYCoord + THIS.nMargin)
         THIS.lstAvailableObjectsAndProcs.Height = ;
              (THIS.edtMarkedCode.Top - (THIS.nMargin * 3))
             
      OTHERWISE
         ?? CHR(7)
      ENDCASE
   ENDPROC
   
   PROC Resize
           
      STORE THIS.Width - (THIS.nMargin*2) TO ;
            THIS.edtMarkedCode.Width, ;
            THIS.lstAvailableObjectsAndProcs.Width
            
      STORE STR(THIS.Width/2)+","+STR(THIS.Width/2) TO ;
           THIS.lstAvailableObjectsAndProcs.ColumnWidths
           
      STORE (THIS.Height - (THIS.nMargin*3))/2 TO ;
            THIS.edtMarkedCode.Height, ;
            THIS.lstAvailableObjectsAndProcs.Height
            
      STORE THIS.lstAvailableObjectsAndProcs.Height + ;
            (THIS.nMargin*2) TO ;
            THIS.edtMarkedCode.Top

   ENDPROC
   
   PROC SetSources(tcTarget)
      
      THIS.cMarkedCodeField = IIF(THISFORMSET.lInProfileMode, ;
                                 "Profiled", "Marked")  
      THIS.lstAvailableObjectsAndProcs.RowSource = tcTarget
      THIS.edtMarkedCode.ControlSource = THIS.cMarkedCodeField
   ENDPROC
   
   PROC SetInitialFormProperties()
      
      WITH THIS.lstAvailableObjectsAndProcs
         .FontName = THISFORMSET.cBaseMonoFontName
         .FontSize = THISFORMSET.nBaseMonoFontSize
         .FontItalic = THISFORMSET.lBaseMonoFontItalic
         .FontBold = THISFORMSET.lBaseMonoFontBold
      ENDWITH

      WITH THIS.edtMarkedCode                                 
         .FontName = THISFORMSET.cBaseMonoFontName
         .FontSize = THISFORMSET.nBaseMonoFontSize
         .FontItalic = THISFORMSET.lBaseMonoFontItalic
         .FontBold = THISFORMSET.lBaseMonoFontBold
      ENDWITH
            
   ENDPROC

   PROC lstAvailableObjectsAndProcs.When
      IF EMPTY(EVAL(THIS.RowSource+"."+THISFORM.cMarkedCodeField))
         THISFORMSET.MarkTargetCoverage(,THIS.RowSource)
      ENDIF
      THISFORM.edtMarkedCode.Refresh()      
      THISFORM.Caption = ;
           ALLTR(EVAL(THIS.RowSource+".HostFile"))
      IF EMPTY(EVAL(THIS.RowSource+".ObjClass"))
         THISFORM.Caption = THISFORM.Caption+ ": ALL Classes & Procs"
      ELSE
         THISFORM.Caption = THISFORM.Caption + ": " + ;
                    ALLTR(EVAL(THIS.RowSource+".ObjClass"))
      ENDIF
   ENDPROC
   
   PROC DESTROY
      * we want the formset to be "defined" by
      * its interface in this case,
      * so we don't want it to go on existing invisibly
      RELEASE THISFORMSET
   ENDPROC

ENDDEFINE

