* COV_ENG2.PRG
* >L<

* Creates a subclass of cov_engine rather
* than the cov_standard, as the classes in the
* COV_SUBS.VCX all do.

* This subclass gets its interface from the GR_STATS.SCX used
* elsewhere as an AddIn on its own, and *also* used
* in Add_Log, where goes to show that you can
* have more than one log being analysed at the same
* time and that, if you like, all their interfaces
* could be different if they were logs that
* concentrated on different things.  

* For example, GR_STATS.SCX, the interface in this program,
* really only deals with the relationship between
* object and line coverage, so it ignores source items
* with no objects (such as DBCs, where all code is placed
* in one record).  

* The point:
* the engine does all the work and the interface(s) can
* be whatever you want.

* There is no significance to the fact that this
* and COV_ENG1.PRG are PRGs rather than in a VCX, except 
* they were both written as quickly as possible <g>.

LPARAMETERS tcFile, tlUnattended, tcAddIn
IF NOT (FILE("COVERAGE.VCX") AND FILE("GR_STATS.SCX")) 
   MESSAGEBOX("You need to either:"+CHR(13)+CHR(13)+ ;
              "*-- build this PRG into an APP or "+;
              CHR(13)+CHR(13)+ ;
              "*-- SET PATH TO <folders with GR_STATS.SCX & COVERAGE.VCX>"+;
              CHR(13)+CHR(13)+ ;
              "before running this demo program!",16,;
              "This Coverage Engine Subclass demo can't run")
   RETURN .F.
ENDIF              

LOCAL lcLibs, loRef
lcLibs = SET("CLASSLIB")
SET CLASSLIB TO COVERAGE ADDITIVE
loRef = CREATEOBJECT("cov_EngineSubClass", tcFile, tlUnattended, tcAddIn)
IF TYPE("loRef.Name") = "C"
   loRef.Activate()
ENDIF   
SET CLASSLIB TO &lcLibs
RETURN

DEFINE CLASS cov_EngineSubClass AS cov_Engine
   oGr_StatsRef = NULL
   lFirstTime = .T.

   PROC Init(tcFile,tlUnattended, tcAddIn)
      LOCAL llReturn
      
      llReturn = DODEFAULT(tcFile,tlUnattended, tcAddIn)
      
      THIS.lInCoverageFrame = .T. && force it for this one
      THIS.CreateForms()
      THIS.oFrame.Show()
      IF (NOT llReturn) OR THIS.lUnattended
         THIS.Release()
         * to get rid of public reference etc;
         * in this limited class,
         * we don't want it lying around
         * whereas in cov_standard and
         * some other times we may 
         * interrogate the object before
         * getting rid of it, in a wrapper program
      ENDIF
   RETURN llReturn AND (NOT THIS.lError)
   
   PROC StandardRightClick
      * so that forms can share this behavior...
      * in this subclass, only the frame will 
      * take advantage of it...
      
      #DEFINE COV_PUBLIC_VARIABLE  _oCoverage
      
      DEFINE POPUP CovShort SHORTCUT RELATIVE FROM MROW(),MCOL()
      DEFINE BAR 1 OF CovShort PROMPT "\<Smart Path"
      DEFINE BAR 2 OF CovShort PROMPT "Show \<Log"

      ON SELECTION BAR 1 OF covshort COV_PUBLIC_VARIABLE.lSmartPath = ;
                                     ! COV_PUBLIC_VARIABLE.lSmartPath
      ON SELECTION BAR 2 OF covshort COV_PUBLIC_VARIABLE.ShowLog()
      
      SET MARK OF BAR 1 OF CovShort TO COV_PUBLIC_VARIABLE.lSmartPath
      ON SELECTION POPUP CovShort DEAC POPUP
      ACTIVATE POPUP CovShort
      RELEASE POPUP CovShort

   ENDPROC
   
   PROC ShowLog
   
      IF TYPE("THIS.frmSource.Name") # "C"
         LOCAL loRef,liSelect, lnTemp
         liSelect = SELECT()
         CREATE CURSOR Temp (onefield m)
         APPEND BLANK
         APPEND MEMO Onefield FROM (THIS.cSourceFile)
         THIS.AddObject("frmSource","MyForm")
         WITH THIS.frmSource
            .Caption = THIS.cSourceFile
            lnTemp =THIS.oFrame.Height
            .Height = (lnTemp* 2)/3
            .Top = lnTemp-.Height
            lnTemp =THIS.oFrame.Width
            .Width = (lnTemp* 2)/3
            .Left = lnTemp-.Width
            .Resize()
           .edtLog.ControlSource = "temp.onefield"
            THIS.Show()
            .Show()
         ENDWITH
         SELECT (liSelect)
      ELSE
        THIS.frmSource.Show()
      ENDIF
   ENDPROC
   
   PROC Activate
      IF TYPE("THIS.oGr_StatsRef.Name") # "C"
         DO FORM GR_STATS.SCX WITH THIS
         THIS.oGr_StatsRef = GR_STATS
         THIS.lFirstTime = .F.
      ENDIF
   ENDPROC

   PROC Deactivate
      IF TYPE("THIS.oGr_StatsRef.Name") # "C" AND ;
         NOT THIS.lFirstTime
         THIS.Release()
         * really this kind of 
         * code should be in the 
         * form QueryUnload
      ENDIF
   ENDPROC 
   
   PROC Destroy
      THIS.oGr_StatsRef = NULL
      IF USED("Temp")
         USE IN Temp
      ENDIF
   ENDPROC

ENDDEFINE
   
   
DEFINE CLASS MyForm AS Form

   ShowWindow = 1

   ADD OBJECT edtLog AS editbox WITH ;
      Readonly = .T., ;
      Left = 0, ;
      Top = 0, ;
      Colorsource = 1
      
   PROC Resize
      THIS.edtLog.Width = THIS.Width
      THIS.edtLog.Height = THIS.Height
   ENDPROC
   
ENDDEFINE

