*==============================================================================
* Program:				NewObject
* Purpose:				Create an object from a specified class
* Author:				Doug Hennig
* Copyright:			(c) 1997 Stonefield Systems Group Inc.
* Last revision:		03/02/98
* Parameters:			tcClass   - the name of the class to instantiate
*						tcLibrary - the name of the class library containing
*							the class
*						tuParm1   - an optional parameter to pass to the new
*							object's Init method
*						tuParm2   - an optional parameter to pass to the new
*							object's Init method
*						tuParm3   - an optional parameter to pass to the new
*							object's Init method
*						tuParm4   - an optional parameter to pass to the new
*							object's Init method
* Returns:				the return value from CREATEOBJECT() (usually an object
*							reference but it could also be .F. if the object
*							failed to instantiate)
* Environment in:		if tcLibrary isn't specified, it must already be opened
*							with SET CLASSLIB (for a VCX) or SET LIBRARY or SET
*							PROCEDURE (for a PRG)
* Environment out:		the object may have been instantiated
*==============================================================================

lparameters tcClass, ;
	tcLibrary, ;
	tuParm1, ;
	tuParm2, ;
	tuParm3, ;
	tuParm4
local lcLibrary, ;
	loObject

* If the library is specified, open it if necessary.

do case
	case type('tcLibrary') <> 'C' or empty(tcLibrary) or ;
		upper(tcLibrary) $ upper(set('CLASSLIB')) or ;
		upper(tcLibrary) $ upper(set('PROCEDURE')) or ;
		upper(tcLibrary) $ upper(set('LIBRARY'))
		lcLibrary = ''
	case file(tcLibrary)
		lcLibrary = upper(tcLibrary)
	otherwise
		lcLibrary = upper(locfile(tcLibrary, ;
			'Visual Class Library:VCX;Program:PRG', tcLibrary))
endcase
do case
	case empty(lcLibrary)
	case right(lcLibrary, 4) = '.VCX'
		set classlib to (lcLibrary) additive
	otherwise
		set procedure to (lcLibrary) additive
endcase

* Create the object.

do case
	case pcount() < 3
		loObject = createobject(tcClass)
	case pcount() = 3
		loObject = createobject(tcClass, @tuParm1)
	case pcount() = 4
		loObject = createobject(tcClass, @tuParm1, @tuParm2)
	case pcount() = 5
		loObject = createobject(tcClass, @tuParm1, @tuParm2, @tuParm3)
	case pcount() = 6
		loObject = createobject(tcClass, @tuParm1, @tuParm2, @tuParm3, ;
			@tuParm4)
endcase

* Release the library if we opened it (and it's still open) and return a
* reference to the object we created.

do case
	case empty(lcLibrary) or not (lcLibrary $ upper(set('CLASSLIB')) or ;
		lcLibrary $ upper(set('PROCEDURE')) or ;
		lcLibrary $ upper(set('LIBRARY')))
	case right(lcLibrary, 4) = '.VCX'
		release classlib (lcLibrary)
	otherwise
		release procedure (lcLibrary)
endcase
return loObject
